@@ -28,16 +28,16 @@ __license__   = "LGPL"
 import pyatspi
 import re
 
-import braille
-import debug
-import eventsynthesizer
-import orca_state
-import settings
+from . import braille
+from . import debug
+from . import eventsynthesizer
+from . import orca_state
+from . import settings
 
-from braille_generator import BrailleGenerator
-from orca_i18n import _         # for gettext support
-from orca_i18n import C_        # to provide qualified translatable strings
-from orca_i18n import ngettext
+from .braille_generator import BrailleGenerator
+from .orca_i18n import _
+from .orca_i18n import C_
+from .orca_i18n import ngettext
 
 # [[[WDW - HACK Regular expression to split strings on whitespace
 # boundaries, which is what we'll use for word dividers instead of
@@ -46,7 +46,7 @@ from orca_i18n import ngettext
 #
 whitespace_re = re.compile(r'(\s+)', re.DOTALL | re.IGNORECASE | re.M)
 
-EMBEDDED_OBJECT_CHARACTER = u'\ufffc'
+EMBEDDED_OBJECT_CHARACTER = '\ufffc'
 
 class Char:
     """Represents a single char of an Accessibility_Text object."""
@@ -100,7 +100,7 @@ class Word:
         self.index = index
         self.startOffset = startOffset
         self.string = string
-        self.length = len(string.decode("UTF-8"))
+        self.length = len(string)
         self.x = x
         self.y = y
         self.width = width
@@ -142,7 +142,7 @@ class Word:
                     # Bug #495303. We can try to correct this.
                     #
                     if len(char):
-                        char = char.decode("UTF-8")[0].encode("UTF-8")
+                        char[0]
                     [x, y, width, height] = text.getRangeExtents(
                         startOffset,
                         startOffset + 1,
@@ -156,7 +156,7 @@ class Word:
                 self.chars = None
             return self.chars
         elif attr.startswith('__') and attr.endswith('__'):
-            raise AttributeError, attr
+            raise AttributeError(attr)
         else:
             return self.__dict__[attr]
 
@@ -180,7 +180,7 @@ class Zone:
 
         self.accessible = accessible
         self.string = string
-        self.length = len(string.decode("UTF-8"))
+        self.length = len(string)
         self.x = x
         self.y = y
         self.width = width
@@ -211,7 +211,7 @@ class Zone:
 
             return self.words
         elif attr.startswith('__') and attr.endswith('__'):
-            raise AttributeError, attr
+            raise AttributeError(attr)
         else:
             return self.__dict__[attr]
 
@@ -236,7 +236,7 @@ class Zone:
         wordAtOffset = None
         offset = 0
         for word in self.words:
-            nextOffset = offset + len(word.string.decode("UTF-8"))
+            nextOffset = offset + len(word.string)
             wordAtOffset = word
             if nextOffset > charOffset:
                 return [wordAtOffset, charOffset - offset]
@@ -298,7 +298,7 @@ class TextZone(Zone):
             offset = self.startOffset
             for string in whitespace_re.split(self.string):
                 if len(string):
-                    endOffset = offset + len(string.decode("UTF-8"))
+                    endOffset = offset + len(string)
                     [x, y, width, height] = text.getRangeExtents(
                         offset,
                         endOffset,
@@ -315,7 +315,7 @@ class TextZone(Zone):
             return self.words
 
         elif attr.startswith('__') and attr.endswith('__'):
-            raise AttributeError, attr
+            raise AttributeError(attr)
         else:
             return self.__dict__[attr]
 
@@ -525,7 +525,7 @@ class Line:
             elif attr == "height":
                 return bounds[3] - bounds[1]
         elif attr.startswith('__') and attr.endswith('__'):
-            raise AttributeError, attr
+            raise AttributeError(attr)
         else:
             return self.__dict__[attr]
 
@@ -577,10 +577,7 @@ class Line:
                 region.brailleOffset = brailleOffset
                 self.brailleRegions.append(region)
 
-                try:
-                    regionString = region.string.decode("UTF-8")
-                except UnicodeEncodeError:
-                    regionString = region.string
+                regionString = region.string
                 brailleOffset += len(regionString)
 
             if not settings.disableBrailleEOL:
@@ -791,7 +788,7 @@ class Context:
         substringStartOffset = startOffset
         substringEndOffset   = startOffset
         unicodeStartOffset   = 0
-        unicodeString = string.decode("UTF-8")
+        unicodeString = string
         #print "LOOKING AT '%s'" % unicodeString
         for i in range(0, len(unicodeString) + 1):
             if (i != len(unicodeString)) \
@@ -840,7 +837,7 @@ class Context:
                     #print " SUBSTRING '%s'" % substring
                     zones.append(TextZone(accessible,
                                           substringStartOffset,
-                                          substring.encode("UTF-8"),
+                                          substring,
                                           clipping[0],
                                           clipping[1],
                                           clipping[2],
@@ -919,16 +916,6 @@ class Context:
                 offset,
                 pyatspi.TEXT_BOUNDARY_LINE_START)
 
-            #NEED TO SKIP OVER EMBEDDED_OBJECT_CHARACTERS
-
-            #print "STRING at %d is (start=%d end=%d): '%s'" \
-            #      % (offset, startOffset, endOffset, string)
-            #if startOffset > offset:
-            #    embedded = text.getText(offset, offset + 1).decode("UTF-8")
-            #    if embedded[0] == \
-            #            orca_state.activeScript.EMBEDDED_OBJECT_CHARACTER:
-            #        offset = startOffset
-
             debug.println(debug.LEVEL_FINEST,
                           "    line at %d is (start=%d end=%d): '%s'" \
                           % (offset, startOffset, endOffset, string))
@@ -1433,54 +1420,54 @@ class Context:
         return lines
 
     def _dumpCurrentState(self):
-        print "line=%d, zone=%d, word=%d, char=%d" \
+        print("line=%d, zone=%d, word=%d, char=%d" \
               % (self.lineIndex,
                  self.zoneIndex,
                  self.wordIndex,
-                 self.zoneIndex)
+                 self.zoneIndex))
 
         zone = self.lines[self.lineIndex].zones[self.zoneIndex]
         text = zone.accessible.queryText()
 
         if not text:
-            print "  Not Accessibility_Text"
+            print("  Not Accessibility_Text")
             return
 
-        print "  getTextBeforeOffset: %d" % text.caretOffset
+        print("  getTextBeforeOffset: %d" % text.caretOffset)
         [string, startOffset, endOffset] = text.getTextBeforeOffset(
             text.caretOffset,
             pyatspi.TEXT_BOUNDARY_WORD_START)
-        print "    WORD_START: start=%d end=%d string='%s'" \
-              % (startOffset, endOffset, string)
+        print("    WORD_START: start=%d end=%d string='%s'" \
+              % (startOffset, endOffset, string))
         [string, startOffset, endOffset] = text.getTextBeforeOffset(
             text.caretOffset,
             pyatspi.TEXT_BOUNDARY_WORD_END)
-        print "    WORD_END:   start=%d end=%d string='%s'" \
-              % (startOffset, endOffset, string)
+        print("    WORD_END:   start=%d end=%d string='%s'" \
+              % (startOffset, endOffset, string))
 
-        print "  getTextAtOffset: %d" % text.caretOffset
+        print("  getTextAtOffset: %d" % text.caretOffset)
         [string, startOffset, endOffset] = text.getTextAtOffset(
             text.caretOffset,
             pyatspi.TEXT_BOUNDARY_WORD_START)
-        print "    WORD_START: start=%d end=%d string='%s'" \
-              % (startOffset, endOffset, string)
+        print("    WORD_START: start=%d end=%d string='%s'" \
+              % (startOffset, endOffset, string))
         [string, startOffset, endOffset] = text.getTextAtOffset(
             text.caretOffset,
             pyatspi.TEXT_BOUNDARY_WORD_END)
-        print "    WORD_END:   start=%d end=%d string='%s'" \
-              % (startOffset, endOffset, string)
+        print("    WORD_END:   start=%d end=%d string='%s'" \
+              % (startOffset, endOffset, string))
 
-        print "  getTextAfterOffset: %d" % text.caretOffset
+        print("  getTextAfterOffset: %d" % text.caretOffset)
         [string, startOffset, endOffset] = text.getTextAfterOffset(
             text.caretOffset,
             pyatspi.TEXT_BOUNDARY_WORD_START)
-        print "    WORD_START: start=%d end=%d string='%s'" \
-              % (startOffset, endOffset, string)
+        print("    WORD_START: start=%d end=%d string='%s'" \
+              % (startOffset, endOffset, string))
         [string, startOffset, endOffset] = text.getTextAfterOffset(
             text.caretOffset,
             pyatspi.TEXT_BOUNDARY_WORD_END)
-        print "    WORD_END:   start=%d end=%d string='%s'" \
-              % (startOffset, endOffset, string)
+        print("    WORD_END:   start=%d end=%d string='%s'" \
+              % (startOffset, endOffset, string))
 
     def setCurrent(self, lineIndex, zoneIndex, wordIndex, charIndex):
         """Sets the current character of interest.
@@ -1640,7 +1627,7 @@ class Context:
                 if zone.words:
                     for wordIndex in range(0, self.wordIndex):
                         regionWithFocus.cursorOffset += \
-                            len(zone.words[wordIndex].string.decode("UTF-8"))
+                            len(zone.words[wordIndex].string)
                 regionWithFocus.cursorOffset += self.charIndex
                 regionWithFocus.repositionCursor()
                 break
