@@ -31,11 +31,11 @@ import pyatspi
 import time
 import unicodedata
 
-import debug
-import keynames
-import orca_state
-import settings
-from orca_i18n import C_
+from . import debug
+from . import keynames
+from . import orca_state
+from . import settings
+from .orca_i18n import C_
 
 KEYBOARD_EVENT     = "keyboard"
 BRAILLE_EVENT      = "braille"
@@ -231,7 +231,7 @@ class KeyboardEvent(InputEvent):
 
         return self.event_string in \
             ["F1", "F2", "F3", "F4", "F5", "F6",
-             "F7","F8", "F9", "F10", "F11", "F12"]
+             "F7", "F8", "F9", "F10", "F11", "F12"]
 
     def isLockingKey(self):
         """Return True if this is a locking key."""
@@ -267,17 +267,7 @@ class KeyboardEvent(InputEvent):
         if orca_state.bypassNextCommand:
             return False
 
-        orcaMods = settings.orcaModifierKeys
-        try:
-            orcaMods = map(lambda x: x.encode('UTF-8'), orcaMods)
-        except (UnicodeDecodeError, UnicodeEncodeError):
-            pass
-
-        string = self.event_string
-        if isinstance(string, unicode):
-            string = string.encode('UTF-8')
-
-        return string in orcaMods
+        return self.event_string in settings.orcaModifierKeys
 
     def isOrcaModified(self):
         """Return True if this key is Orca modified."""
@@ -296,14 +286,13 @@ class KeyboardEvent(InputEvent):
         if self.event_string in ["space", " "]:
             return True
 
-        unicodeString = self.event_string.decode("UTF-8")
-        if not len(unicodeString) == 1:
+        if not len(self.event_string) == 1:
             return False
 
-        if unicodeString.isalnum() or unicodeString.isspace():
+        if self.event_string.isalnum() or self.event_string.isspace():
             return True
 
-        return unicodedata.category(unicodeString)[0] in ('P', 'S')
+        return unicodedata.category(self.event_string)[0] in ('P', 'S')
 
     def isPressedKey(self):
         """Returns True if the key is pressed"""
@@ -318,7 +307,7 @@ class KeyboardEvent(InputEvent):
         if not self.isPrintableKey():
             return False
 
-        if settings.learnModeEnabled:
+        if orca_state.learnModeEnabled:
             return False
 
         script = orca_state.activeScript
@@ -382,6 +371,9 @@ class KeyboardEvent(InputEvent):
             return False
         if not orca_state.lastInputEvent:
             return False
+        if isinstance(self, MouseButtonEvent) \
+           or isinstance(orca_state.lastInputEvent, MouseButtonEvent):
+            return False
         if self.hw_code == orca_state.lastInputEvent.hw_code \
            and self.type == orca_state.lastInputEvent.type:
             return True
@@ -393,7 +385,7 @@ class KeyboardEvent(InputEvent):
         if we presented the event. False if there was some reason the event
         was not worthy of presentation."""
 
-        if not settings.learnModeEnabled:
+        if not orca_state.learnModeEnabled:
             if self.shouldEcho == False or self.isOrcaModified():
                 return False
 
@@ -420,7 +412,7 @@ class KeyboardEvent(InputEvent):
                 else:
                     if not self.event_string in [string, 'space']:
                         return False
-            elif not (settings.learnModeEnabled or self.isLockingKey()):
+            elif not (orca_state.learnModeEnabled or self.isLockingKey()):
                 return False
 
         elif not self.isPressedKey():
@@ -484,14 +476,18 @@ class InputEventHandler:
 
     def __eq__(self, other):
         """Compares one input handler to another."""
+
+        if not other:
+            return False
+
         return (self.function == other.function)
 
     def processInputEvent(self, script, inputEvent):
-        """Processes an input event.  If settings.learnModeEnabled is True,
-        this will merely report the description of the input event to braille
-        and speech.  If settings.learnModeEnabled is False, this will call the
-        function bound to this InputEventHandler instance, passing the
-        inputEvent as the sole argument to the function.
+        """Processes an input event.  If learnModeEnabled is True,
+        this will merely present the description of the input event via
+        If learnModeEnabled is False, this will call the function bound
+        to this InputEventHandler instance, passing the inputEvent as
+        the sole argument to the function.
 
         This function is expected to return True if it consumes the
         event; otherwise it is expected to return False.
@@ -504,7 +500,7 @@ class InputEventHandler:
 
         consumed = False
 
-        if settings.learnModeEnabled and self._learnModeEnabled:
+        if orca_state.learnModeEnabled and self._learnModeEnabled:
             if self.description:
                 script.presentMessage(self.description)
                 consumed = True
