@@ -41,20 +41,27 @@ __license__   = "LGPL"
 
 import pyatspi
 
-import braille_generator
-import debug
-import flat_review
-import formatting
-import label_inference
-import keybindings
-import orca_state
-import script_utilities
-import settings
-import speech_generator
-import structural_navigation
-import where_am_I
-import bookmarks
-import tutorialgenerator
+from . import braille_generator
+from . import debug
+from . import event_manager
+from . import flat_review
+from . import formatting
+from . import label_inference
+from . import keybindings
+from . import orca_state
+from . import script_manager
+from . import script_utilities
+from . import settings
+from . import settings_manager
+from . import speech_generator
+from . import structural_navigation
+from . import where_am_I
+from . import bookmarks
+from . import tutorialgenerator
+
+_eventManager = event_manager.getManager()
+_scriptManager = script_manager.getManager()
+_settingsManager = settings_manager.getManager()
 
 class Script:
     """The specific focus tracking scripts for applications.
@@ -192,7 +199,7 @@ class Script:
         braille.py) that match the given inputEventHandler passed.
         """
         return [command
-                for command, handler in self.brailleBindings.iteritems()
+                for command, handler in list(self.brailleBindings.items())
                 if inputEventHandler == handler]
 
     def getFormatting(self):
@@ -312,15 +319,31 @@ class Script:
         """
         return
 
+    def registerEventListeners(self):
+        """Tells the event manager to start listening for all the event types
+        of interest to the script.
+
+        Arguments:
+        - script: the script.
+        """
+
+        _eventManager.registerScriptListeners(self)
+
+    def deregisterEventListeners(self):
+        """Tells the event manager to stop listening for all the event types
+        of interest to the script.
+
+        Arguments:
+        - script: the script.
+        """
+
+        _eventManager.deregisterScriptListeners(self)
+
     def getSettings(self):
         """Returns the settings associated with this script, regardless of
         whether or not the script is active.
         """
 
-        import orca
-        _scriptManager = getattr(orca, '_scriptManager')
-        _settingsManager = getattr(orca, '_settingsManager')
-
         scriptSettings = settings
         if orca_state.activeScript != self:
             name = _scriptManager.getModuleName(self.app)
@@ -358,11 +381,6 @@ class Script:
         - event: the Event
         """
 
-        if not event.source:
-            msg = 'script.processObjectEvent: event.source went away'
-            debug.println(debug.LEVEL_FINE, msg)
-            return
-
         try:
             role = event.source.getRole()
         except (LookupError, RuntimeError):
@@ -399,7 +417,7 @@ class Script:
         # to the order in which they were added.  So...we need to do something
         # different here.  Logged as bugzilla bug 319781.]]]
         #
-        for key in self.listeners.keys():
+        for key in list(self.listeners.keys()):
             if event.type.startswith(key):
                 self.listeners[key](event)
 
