@@ -30,18 +30,18 @@ import pyatspi
 import orca.orca as orca
 import orca.debug as debug
 import orca.scripts.default as default
+import orca.settings_manager as settings_manager
 import orca.orca_state as orca_state
 import orca.speech as speech
 import orca.scripts.toolkits.Gecko as Gecko
-
 from orca.orca_i18n import _
 
-from formatting import Formatting
-from speech_generator import SpeechGenerator
-from script_utilities import Utilities
-import script_settings
+from .formatting import Formatting
+from .speech_generator import SpeechGenerator
+from .script_utilities import Utilities
+from . import script_settings
 
-_settingsManager = getattr(orca, '_settingsManager')
+_settingsManager = settings_manager.getManager()
 
 ########################################################################
 #                                                                      #
@@ -485,7 +485,16 @@ class Script(Gecko.Script):
                 default.Script.onTextInserted(self, event)
                 return
 
-            if event.type.endswith("system") and event.any_data:
+            # Mozilla cannot seem to get their ":system" suffix right
+            # to save their lives, so we'll add yet another sad hack.
+            try:
+                text = event.source.queryText()
+            except:
+                hasSelection = False
+            else:
+                hasSelection = text.getNSelections() > 0
+
+            if (hasSelection or event.type.endswith("system")) and event.any_data:
                 # The autocompleted address may start with the name,
                 # or it might start with the text typed by the user
                 # followed by ">>" followed by the address. Therefore
