@@ -34,18 +34,19 @@ import orca.scripts.default as default
 import orca.input_event as input_event
 import orca.orca as orca
 import orca.settings as settings
+import orca.settings_manager as settings_manager
 import orca.speechserver as speechserver
 import orca.orca_state as orca_state
 import orca.speech as speech
 from orca.orca_i18n import _
 
-import script_settings
-from structural_navigation import StructuralNavigation
-from braille_generator import BrailleGenerator
-from speech_generator import SpeechGenerator
-from script_utilities import Utilities
+from . import script_settings
+from .structural_navigation import StructuralNavigation
+from .braille_generator import BrailleGenerator
+from .speech_generator import SpeechGenerator
+from .script_utilities import Utilities
 
-_settingsManager = getattr(orca, '_settingsManager')
+_settingsManager = settings_manager.getManager()
 
 ########################################################################
 #                                                                      #
@@ -107,6 +108,36 @@ class Script(default.Script):
                 #
                 _("Speaks entire document."))
 
+        self.inputEventHandlers["panBrailleLeftHandler"] = \
+            input_event.InputEventHandler(
+                Script.panBrailleLeft,
+                # Translators: a refreshable braille display is an
+                # external hardware device that presents braille
+                # character to the user.  There are a limited number
+                # of cells on the display (typically 40 cells).  Orca
+                # provides the feature to build up a longer logical
+                # line and allow the user to press buttons on the
+                # braille display so they can pan left and right over
+                # this line.
+                #
+                _("Pans the braille display to the left."),
+                False) # Do not enable learn mode for this action
+
+        self.inputEventHandlers["panBrailleRightHandler"] = \
+            input_event.InputEventHandler(
+                Script.panBrailleRight,
+                # Translators: a refreshable braille display is an
+                # external hardware device that presents braille
+                # character to the user.  There are a limited number
+                # of cells on the display (typically 40 cells).  Orca
+                # provides the feature to build up a longer logical
+                # line and allow the user to press buttons on the
+                # braille display so they can pan left and right over
+                # this line.
+                #
+                _("Pans the braille display to the right."),
+                False) # Do not enable learn mode for this action
+
     def getToolkitKeyBindings(self):
         """Returns the toolkit-specific keybindings for this script."""
 
@@ -396,13 +427,13 @@ class Script(default.Script):
 
         phrase = self.utilities.substring(obj, startOffset, endOffset)
         if len(phrase) and phrase != "\n":
-            if phrase.decode("UTF-8").isupper():
+            if phrase.isupper():
                 voice = self.voices[settings.UPPERCASE_VOICE]
             else:
                 voice = self.voices[settings.DEFAULT_VOICE]
 
             phrase = self.utilities.adjustForRepeats(phrase)
-            links = filter(lambda x: x.getRole() == pyatspi.ROLE_LINK, obj)
+            links = [x for x in obj if x.getRole() == pyatspi.ROLE_LINK]
             if links:
                 phrase = self.utilities.adjustForLinks(obj, phrase, startOffset)
             speech.speak(phrase, voice)
@@ -447,9 +478,12 @@ class Script(default.Script):
         if not self.utilities.isWebKitGtk(orca_state.locusOfFocus):
             return False
 
+        states = orca_state.locusOfFocus.getState()
+        if states.contains(pyatspi.STATE_EDITABLE):
+            return False
+
         role = orca_state.locusOfFocus.getRole()
         if role in doNotHandleRoles:
-            states = orca_state.locusOfFocus.getState()
             if role == pyatspi.ROLE_LIST_ITEM:
                 return not states.contains(pyatspi.STATE_SELECTABLE)
 
@@ -482,13 +516,59 @@ class Script(default.Script):
 
         index = -1
         text = child.queryText()
-        for i in xrange(text.characterCount):
+        for i in range(text.characterCount):
             if text.setCaretOffset(i):
                 index = i
                 break
 
         return child, index
 
+    def panBrailleLeft(self, inputEvent=None, panAmount=0):
+        """In document content, we want to use the panning keys to browse the
+        entire document.
+        """
+
+        if self.flatReviewContext \
+           or not self.isBrailleBeginningShowing() \
+           or not self.utilities.isWebKitGtk(orca_state.locusOfFocus):
+            return default.Script.panBrailleLeft(self, inputEvent, panAmount)
+
+        obj = self.utilities.findPreviousObject(orca_state.locusOfFocus)
+        orca.setLocusOfFocus(None, obj, notifyScript=False)
+        self.updateBraille(obj)
+
+        # Hack: When panning to the left in a document, we want to start at
+        # the right/bottom of each new object. For now, we'll pan there.
+        # When time permits, we'll give our braille code some smarts.
+        while self.panBrailleInDirection(panToLeft=False):
+            pass
+        self.refreshBraille(False)
+
+        return True
+
+    def panBrailleRight(self, inputEvent=None, panAmount=0):
+        """In document content, we want to use the panning keys to browse the
+        entire document.
+        """
+
+        if self.flatReviewContext \
+           or not self.isBrailleEndShowing() \
+           or not self.utilities.isWebKitGtk(orca_state.locusOfFocus):
+            return default.Script.panBrailleRight(self, inputEvent, panAmount)
+
+        obj = self.utilities.findNextObject(orca_state.locusOfFocus)
+        orca.setLocusOfFocus(None, obj, notifyScript=False)
+        self.updateBraille(obj)
+
+        # Hack: When panning to the right in a document, we want to start at
+        # the left/top of each new object. For now, we'll pan there. When time
+        # permits, we'll give our braille code some smarts.
+        while self.panBrailleInDirection(panToLeft=True):
+            pass
+        self.refreshBraille(False)
+
+        return True
+
     def sayAll(self, inputEvent):
         """Speaks the contents of the document beginning with the present
         location.  Overridden in this script because the sayAll could have
@@ -530,12 +610,21 @@ class Script(default.Script):
         spoken and acss is an ACSS instance for speaking the text.
         """
 
+        if not obj:
+            return
+
+        if obj.getRole() == pyatspi.ROLE_LINK:
+            obj = obj.parent
+
         document = utils.findAncestor(
             obj, lambda x: x.getRole() == pyatspi.ROLE_DOCUMENT_FRAME)
+        if not document:
+            return
+
         allTextObjs = utils.findAllDescendants(
             document, lambda x: 'Text' in utils.listInterfaces(x))
         allTextObjs = allTextObjs[allTextObjs.index(obj):len(allTextObjs)]
-        textObjs = filter(lambda x: x.parent not in allTextObjs, allTextObjs)
+        textObjs = [x for x in allTextObjs if x.parent not in allTextObjs]
         if not textObjs:
             return
 
@@ -583,7 +672,7 @@ class Script(default.Script):
         else:
             linkCount = hypertext.getNLinks()
             links = [hypertext.getLink(x) for x in range(linkCount)]
-            if filter(lambda l: l.startIndex <= offset <= l.endIndex, links):
+            if [l for l in links if l.startIndex <= offset <= l.endIndex]:
                 return
 
         text.setCaretOffset(offset)
@@ -601,13 +690,53 @@ class Script(default.Script):
         where the caret is.
         """
 
-        # TODO - JD: Piece together the full line for braille
-
         textLine = default.Script.getTextLineAtCaret(self, obj, offset)
-        string = textLine[0].decode('UTF-8')
-        if string.find(self.EMBEDDED_OBJECT_CHARACTER) == -1:
+        string = textLine[0]
+        if string and string.find(self.EMBEDDED_OBJECT_CHARACTER) == -1 \
+           and obj.getState().contains(pyatspi.STATE_FOCUSED):
             return textLine
 
         textLine[0] = self.utilities.displayedText(obj)
+        try:
+            text = obj.queryText()
+        except:
+            pass
+        else:
+            textLine[1] = min(textLine[1], text.characterCount)
 
         return textLine
+
+    def updateBraille(self, obj, extraRegion=None):
+        """Updates the braille display to show the given object.
+
+        Arguments:
+        - obj: the Accessible
+        - extra: extra Region to add to the end
+        """
+
+        if not obj:
+            return
+
+        if not self.utilities.isWebKitGtk(obj) \
+           or (not self.utilities.isInlineContainer(obj) \
+               and not self.utilities.isTextListItem(obj)):
+            default.Script.updateBraille(self, obj, extraRegion)
+            return
+
+        brailleLine = self.getNewBrailleLine(clearBraille=True, addLine=True)
+        for child in obj:
+            if not self.utilities.onSameLine(child, obj[0]):
+                break
+            [regions, fRegion] = self.brailleGenerator.generateBraille(child)
+            self.addBrailleRegionsToLine(regions, brailleLine)
+
+        if not brailleLine.regions:
+            [regions, fRegion] = self.brailleGenerator.generateBraille(
+                obj, role=pyatspi.ROLE_PARAGRAPH)
+            self.addBrailleRegionsToLine(regions, brailleLine)
+            self.setBrailleFocus(fRegion)
+
+        if extraRegion:
+            self.addBrailleRegionToLine(extraRegion, brailleLine)
+
+        self.refreshBraille()
